<?php

/**
 * ServicesController
 *
 * web-DPO : Outil de gestion de vos traitements dans le cadre de la 
 * réglementation relative à la protection des données personnelles (RGPD)
 * 
 * Copyright (c) Libriciel SCOP (https://www.libriciel.fr/)
 *
 * Licensed under the GNU Affero General Public License version 3 License - AGPL v3
 * For full copyright and license information, please see the "LICENSE" file.
 * Redistributions of files must retain the above copyright notice.
 * 
 * @copyright   Copyright (c) Libriciel SCOP (https://www.libriciel.fr/)
 * @link        https://www.libriciel.fr/web-dpo/
 * @since       web-DPO v1.0.0
 * @license     [GNU Affero General Public License version 3](http://www.gnu.org/licenses/agpl-3.0.html) - AGPL v3
 * @version     v1.0.0
 * @package     App.Controller
 */

App::uses('ListeDroit', 'Model');

class ServicesController extends AppController {

    public $uses = array(
        'Service',
        'OrganisationUserService'
    );

    /**
     * Vérification de l'accès aux actions en fonction du profil de l'utilisateur connecté.
     * Les vérifications de l'accès aux enregistrements se font dans les méthodes d'actions.
     */
    public function beforeFilter() {
        parent::beforeFilter();

        $action = Inflector::underscore($this->request->params['action']);
        $listeDroitIndex = [ListeDroit::CREER_SERVICE, ListeDroit::MODIFIER_SERVICE, ListeDroit::SUPPRIMER_SERVICE];

        if ($action === 'add') {
            $this->Droits->assertAuthorized([ListeDroit::CREER_SERVICE]);
        } elseif ($action === 'edit') {
            $this->Droits->assertAuthorized([ListeDroit::MODIFIER_SERVICE]);
        } elseif ($action === 'index') {
            $this->Droits->assertAuthorized($listeDroitIndex);
        } elseif ($action === 'delete') {
            $this->Droits->assertAuthorized([ListeDroit::SUPPRIMER_SERVICE]);
        } else {
            throw new ForbiddenException(__d('default', 'default.flasherrorPasDroitPage'));
        }
    }

    /**
     * @access public
     * @created 18/06/2015
     * @version V1.0.0
     */
    public function index() {
        $this->set('title', __d('service', 'service.titreService') . $this->Session->read('Organisation.raisonsociale'));

        $query = [
            'fields' => array_merge(
                $this->Service->fields(),
                [
                    $this->Service->vfLinkedUsersCount()
                ]
            ),
            'conditions' => [
                'organisation_id' => $this->Session->read('Organisation.id')
            ],
            'order' => 'libelle ASC'
        ];
        $this->set('serv', $this->Service->find('all', $query));
    }

    /**
     * @access public
     * @created 18/06/2015
     * @version V1.0.0
     */
    public function add() {
        $this->set('title', __d('service', 'service.titreAjouterService'));

        if ($this->request->is('post')) {
            if ('Cancel' === Hash::get($this->request->data, 'submit')) {
                $this->redirect($this->Referers->get());
            }
            
            $success = true;
            $this->Service->begin();

            $this->Service->create($this->request->data);
            $success = $success && false !== $this->Service->save(null, ['atomic' => false]);

            if ($success == true) {
                $this->Service->commit();
                $this->Session->setFlash(__d('service', 'service.flashsuccessServiceEnregistrer'), 'flashsuccess');
                
                $this->redirect($this->Referers->get());
            } else {
                $this->Service->rollback();
                $this->Session->setFlash(__d('service', 'service.flasherrorErreurEnregistrementService'), 'flasherror');
            }
        }
    }

    /**
     * @param int|null $id
     *
     * @access public
     * @created 18/06/2015
     * @version V1.0.0
     */
    public function edit($id = null) {
        $this->Droits->assertRecordAuthorized('Service', $id, ['superadmin' => true]);

        $this->set('title', __d('service', 'service.titreModifierrService'));

        if ($this->request->is('post') || $this->request->is('put')) {
            if ('Cancel' === Hash::get($this->request->data, 'submit')) {
                $this->redirect($this->Referers->get());
            }
            
            $success = true;
            $this->Service->begin();
            
            $this->request->data['Service']['id'] = $id;
            
            $this->Service->create($this->request->data);
            $success = $success && false !== $this->Service->save(null, ['atomic' => false]);

            if ($success == true) {
                $this->Service->commit();
                $this->Session->setFlash(__d('service', 'service.flashsuccessServiceEnregistrer'), "flashsuccess");
                
                $this->redirect($this->Referers->get());
            } else {
                $this->Service->rollback();
                $this->Session->setFlash(__d('service', 'service.flasherrorErreurEnregistrementService'), 'flasherror');
            }
        } else {
            $this->request->data = $this->Service->findById($id);
        }
    }

    /**
     * @param int|null $id
     *
     * @access public
     * @created 18/06/2015
     * @version V1.0.0
     */
    public function delete($id = null) {
        $this->Droits->assertRecordAuthorized('Service', $id, ['superadmin' => true]);

        // Le service ne doit être lié à aucun utilisateur
        $query = [
            'fields' => [$this->Service->vfLinkedUsersCount()],
            'conditions' => ['Service.id' => $id]
        ];
        $service = $this->Service->find('first', $query);

        if (Hash::get($service, 'Service.users_count') > 0) {
            throw new ForbiddenException(__d('default', 'default.flasherrorPasDroitPage'));
        }

        $this->set('title', 'Supprimer un service');

        $this->Service->begin();
        if ($this->Service->delete($id, false) == true) {
            $this->Service->commit();
            $this->Session->setFlash(__d('service', 'service.flashsuccessServiceSupprimer'), 'flashsuccess');
        } else {
            $this->Service->rollback();
            $this->Session->setFlash(__d('default', 'default.flasherrorErreurSupprimerService'), 'flasherror');
        }

        $this->redirect($this->Referers->get());
    }

}
