<?php

/**
 * Workflow
 *
 * web-DPO : Outil de gestion de vos traitements dans le cadre de la
 * réglementation relative à la protection des données personnelles (RGPD)
 *
 * Copyright (c) Libriciel SCOP (https://www.libriciel.fr/)
 *
 * Licensed under the GNU Affero General Public License version 3 License - AGPL v3
 * For full copyright and license information, please see the "LICENSE" file.
 * Redistributions of files must retain the above copyright notice.
 *
 * @copyright   Copyright (c) Libriciel SCOP (https://www.libriciel.fr/)
 * @link        https://www.libriciel.fr/web-dpo/
 * @since       web-DPO v1.0.0
 * @license     [GNU Affero General Public License version 3](http://www.gnu.org/licenses/agpl-3.0.html) - AGPL v3
 * @version     v2.1.0
 * @package     App.Model
 */

App::uses('AppModel', 'Model');
App::uses('XmlWorkflowConverter', 'WrapperFlowable.Utility');
App::uses('WrapperFactory', 'WrapperFlowable.Utility/Api');
App::uses('FlowableWrapper', 'WrapperFlowable.Utility/Api');

class Workflow extends AppModel {

    public $name = 'Workflow';

    public $validationDomain = 'validation';

    const NO_USER = 'NO_USER';
    const NO_GROUP = 'NO_GROUP';

    /**
     * Règles de validation supplémentaires.
     *
     * @var array
     */
    public $validate = [
        'libelle' => [
            'isUniqueMultiple' => [
                'rule' => ['isUniqueMultiple', ['name', 'organisation_id']],
                'message' => 'validation.valeurDejaUtilisee'
            ]
        ],
        'organisation_id' => [
            'isUniqueMultiple' => [
                'rule' => ['isUniqueMultiple', ['name', 'organisation_id']],
                'message' => 'validation.valeurDejaUtilisee'
            ]
        ]
    ];

    /**
     * belongsTo associations
     *
     * @var array
     */
    public $belongsTo = [
        'Organisation' => [
            'className' => 'Organisation',
            'foreignKey' => 'organisation_id'
        ]
    ];

    /**
     * Conversion du json en fichier bpmn20.xml
     * Enregistrement du fichier sur le disque
     *
     * @param array $circuit Nom du circuit et nom des étapes du Workflows
     * @return array|bool Retourne un tableau avec le nom et le fichier xml pour l'envoie à flowable ou false en cas
     * d'erreur
     *
     * @access public
     * @created 26/03/2019
     * @version V0.0.9
     */
    public function convertJsonToXml($circuit)
    {
        $organisation_id = $circuit['organisation_id'];

        foreach ($circuit['steps'] as &$step) {
            $step['users'] = implode(',', $step['users']);
            if (empty($step['users'])) {
                $step['users'] = self::NO_USER;
            }
            if (empty($step['groups'])) {
                $step['groups'] = self::NO_GROUP;
            }
        }

        $xml_data = XmlWorkflowConverter::convertJsonToXml($circuit);

        // Nécessaire pour le dépot manuel du fichier .bpmn.xml sur l'interface de flowable
//        $this->addBpmndiDiagram($circuit["name"], $xml_data);

        $folder = CHEMIN_WORKFLOWS;
        if (!file_exists($folder)) {
            // On verifie si le dossier file existe. Si c'est pas le cas on le cree
            create_arborescence_files();
        }

        $microtime = time() . '_' . substr(explode(' ', microtime())[0], 2);
        $nameFile = $microtime . '.bpmn20.xml';
        $pathFile = $folder . DS . $nameFile;

        file_put_contents($pathFile, $xml_data->saveXml());

        if (file_exists($pathFile)) {
            $dataSendToFlowable = [
                'deploymentKey' => $circuit['name'],
                'deploymentName' => $circuit['name'],
                'file' => [
                    'name' => $nameFile,
                    'type' => mime_content_type($pathFile),
                    'tmp_name' => $pathFile,
                ],
                'organisation_id' => $organisation_id,
            ];

            return ($dataSendToFlowable);
        }

        return false;
    }

    public function createDeployment(array $data): array
    {
        $organisation_id = $data['organisation_id'];

        $folder = TMP . $organisation_id;

        if (!file_exists($folder)) {
            umask(0);
            mkdir($folder, 0777, true);
        }

        try {
            copy($data['file']['tmp_name'], $folder . DS . $data['file']['name']);
            $data['file']['path'] = $folder . DS . $data['file']['name'];
        } catch (Exception $exception) {
            CakeLog::write('error', $exception->getMessage(), 'flowableErrors');
            throw $exception;
        }

        //create the deployment using flowable
        /* @var $wrapper FlowableWrapper */
        $wrapper = WrapperFactory::createWrapper();
        $response = $wrapper->createDeployment($data);
        debug("TROTRO");
        die;

        //Generate the deployment to save it into the webActes database.
        $deployment = $this->generateDeployment($response, $data);

        return $deployment;
    }

    private function generateDeployment($deploymentRequested, $data): array
    {
        return [
            'process_definition_key' => $deploymentRequested['process_definition_key'],
            'organisation_id' => $data['organisation_id'],
            'file' => $data['file']['name'],
        ];
    }
}
