<?php

use Symfony\Component\Mime\Part\DataPart;
use Symfony\Component\Mime\Part\Multipart\FormDataPart;
use Symfony\Component\HttpClient\HttpClient;

App::uses('HttpSocket', 'Network/Http');

/**
 * The class FlowableShell makes it possible to initialiaze a Flowable instance with a deployment or get the list of
 * deployments.
 *
 * @package WrapperFlowable\Shell
 */
class FlowableDeploymentsShell extends AppShell
{
    /**
     * Returns the deployment URL with respect to the parameters and the configuration.
     *
     * @return string
     */
    protected function _url()
    {
        $format = '%s://%s:%s/flowable-rest/service/repository/deployments';

        return sprintf($format, $this->param('scheme'), $this->param('host'), $this->param('port'));
    }

    /**
     * Returns the flowable auth with respect to the parameters and the configuration.
     *
     * @return array
     */
    protected function _auth()
    {
        return [
            'username' => $this->param('username'),
            'password' => $this->param('password'),
        ];
    }

    /**
     * Add a deployment to the flowable server with a given deployment key, name and file.
     *
     * @throws \Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface
     * @throws \Symfony\Contracts\HttpClient\Exception\DecodingExceptionInterface
     * @throws \Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface
     * @throws \Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface
     * @throws \Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface
     *
     * @return void
     */
    public function init()
    {
        $handle = fopen($this->param('file'), 'r');
        if (empty($handle) === true) {
            $success = false;
            $this->err(sprintf('Could not read file %s', $this->param('file')));
        } else {
            $formFields = [
                'file' => DataPart::fromPath($this->param('file')),
            ];
            $formData = new FormDataPart($formFields);

            $client = HttpClient::create([
                'auth_basic' => $this->_auth()
            ]);

            $url = $this->_url();
            $response = $client->request('POST', $url, [
                'headers' => $formData->getPreparedHeaders()->toArray(),
                'query' => [
                    'deploymentKey' => $this->param('key'),
                    'deploymentName' => $this->param('name'),
                ],
                'body' => $formData->bodyToString(),
            ]);

            if ($response->getStatusCode() === 201) {
                $success = true;
                $deployment = $response->toArray();
                $msgid = '<success>Successfully</success> created a deployment with id <info>%s</info> and name <info>%s</info> at <info>%s</info>';
                $this->out(sprintf($msgid, $deployment['id'], $deployment['name'], $deployment['deploymentTime']));
            } else {
                $success = false;
                $msgid = 'Got reponse code %d from %s (%s)';
                $this->err(sprintf($msgid, $response->getStatusCode(), $url, $response->getContent()));
            }

            if (empty($handle) === false) {
                fclose($handle);
            }
        }

        $this->_stop($success ? static::QUIET : static::CODE_ERROR);
    }

    /**
     * Prints the list of deployments available on the flowable server.
     *
     * @throws \Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface
     * @throws \Symfony\Contracts\HttpClient\Exception\DecodingExceptionInterface
     * @throws \Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface
     * @throws \Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface
     * @throws \Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface
     *
     * @return void
     */
    public function index()
    {
        $client = HttpClient::create([
            'auth_basic' => $this->_auth()
        ]);

        $url = $this->_url();

        $response = $client->request('GET', $url, [
            'query' => [
                'sort' => 'name'
            ],
        ]);

        if ($response->getStatusCode() === 200) {
            $success = true;
            $deployments = $response->toArray();

            $data = [['Id', 'Name', 'Created']];
            foreach ($deployments['data'] as $deployment) {
                $data[] = [
                    $deployment['id'],
                    $deployment['name'],
                    $deployment['deploymentTime'],
                ];
            }

            $msgstr = sprintf(
                'Showing <info>%d</info> records out of <info>%d</info> total, starting on record <info>%d</info>, ending on <info>%d</info>',
                min($deployments['total'], $deployments['size']),
                $deployments['total'],
                $deployments['start'],
                min($deployments['start'] + $deployments['size'], $deployments['total'])
            );
            $this->out($msgstr);
            $this->helper('Table')->output($data);
        } else {
            $success = false;
            $msgid = 'Got reponse code %d from %s (%s)';
            $this->err(sprintf($msgid, $response->getStatusCode(), $url, $response->getContent()));
        }

        $this->_stop($success ? static::QUIET : static::CODE_ERROR);
    }

    /**
     * Complete the option parser for this shell.
     *
     */
    public function getOptionParser()
    {
        $parser = parent::getOptionParser();

        $parser->description(__('This shell makes it possible to initialiaze a Flowable instance with a deployment or get the list of deployments'));

        $parser->addSubcommands(
            [
                'init' => [
                    'help' => __('Add a deployment to the flowable server with a given deployment key, name and file'),
                ],
                'index' => [
                    'help' => __('Prints the list of deployments available on the flowable server.'),
                ],
            ]
        );

        $parser->addOptions(
            [
                'file' => [
                    'help' => 'The path to the file used for the initial deployment',
                    'short' => 'f',
                    'default' => APP . 'Plugin/WrapperFlowable/validation_simple.bpmn20.xml',
                ],
                'host' => [
                    'help' => 'The host name for the flowable service',
                    'short' => 'h',
                    'default' => Configure::read('Flowable.host')
                ],
                'key' => [
                    'help' => 'The flowable deployment key',
                    'short' => 'k',
                    'default' => 'validation_simple',
                ],
                'name' => [
                    'help' => 'The flowable deployment name',
                    'short' => 'n',
                    'default' => 'validationSimple',
                ],
                'password' => [
                    'help' => 'The password for the flowable service',
                    'short' => 'p',
                    'default' => Configure::read('Flowable.auth.password')
                ],
                'port' => [
                    'help' => 'The port for the flowable service',
                    'short' => 'P',
                    'default' => Configure::read('Flowable.port')
                ],
                'scheme' => [
                    'help' => 'The scheme for the flowable service',
                    'short' => 's',
                    'default' => 'http',
                    'choices' => ['http', 'https'],
                ],
                'username' => [
                    'help' => 'The user name for the flowable service',
                    'short' => 'u',
                    'default' => Configure::read('Flowable.auth.username'),
                ],
            ]
        );

        return $parser;
    }
}
